﻿var ReadingModeViewer = (function (module) {
    "use strict";
    var timeToReadText = " ";
    var numViewport = 0;
    var articleDirection = "ltr";
    var columnWidth = 33; 
    var remBaseValue = 16; 
    var paddingToScroll = 100 + 35 + 200; 
    var viewportMarginLeft = 60; 
    var viewportMarginRight = 60; 
    var SCRIPT_EXCEPTION_OCCURED = -1;
    var SCRIPT_PASSED = 0;
    var SCRIPT_FAILED = 1;
    var scrollAllowed = true;
    var minWidthForMultiColumn = 2 * columnWidth * remBaseValue;
    var isPaginatedReadingViewInitialized = false;

    
    module.__proto__.adjustTablesLayout = function () {
        var allTables = document.getElementsByTagName("table");
        for (var id = 0; id < allTables.length; id++) {
            var currentTable = allTables[id];
            currentTable.setAttribute("responsive", "true");
            var thData = new Array();
            var rows = currentTable.querySelectorAll("tr");
            if (rows.length > 0) {
                var firstRow = rows[0];
                firstRow.setAttribute("responsive", "true");
                var dataCount = 0;
                for (var idx = 0; idx < firstRow.children.length; idx++) {
                    var currentHeader = firstRow.children[idx];
                    thData.push(currentHeader.innerText);
                    dataCount++;
                }

                for (var index = 1; index < rows.length; index++) {
                    for (var j = 0; (j < rows[index].children.length) && (j < dataCount); j++) {
                        var currentChild = rows[index].children[j];
                        currentChild.setAttribute("data-th", thData[j]);
                    }
                }
            }
        }
    }

    
    module.__proto__.getNewViewportDiv = function () {
        var viewportDiv = document.createElement("div");
        viewportDiv.className = "viewportContainer";
        viewportDiv.style.width = document.documentElement.clientWidth + "px";
        return viewportDiv;
    }

    
    module.__proto__.getNewRegion = function (regionWidth) {
        
        var divElement = document.createElement("div");
        divElement.className = "region";
        divElement.style.width = regionWidth + "px";
        return divElement;
    }

    
    module.__proto__.generateNewRegions = function () {
        var browserWidth = document.documentElement.clientWidth;
        var columnWidthInPx = columnWidth * remBaseValue;
        var newRegion = getNewRegion(columnWidthInPx);
        var colContainer = getNewViewportDiv();
        var mainCont = document.getElementById("regionContainer");
        var viewportIdx = 0;

        if (articleDirection === "rtl") {
            colContainer.style.left = -1 * document.documentElement.clientWidth + "px";
        }
        else {
            colContainer.style.left = document.documentElement.clientWidth + "px";
        }

        colContainer.appendChild(newRegion);
        mainCont.appendChild(colContainer);
        var availableBrowserWidth = browserWidth - parseInt(window.getComputedStyle(colContainer, null).getPropertyValue("padding-right")) - parseInt(window.getComputedStyle(colContainer, null).getPropertyValue("padding-left"));
        var availableRegionWidth = columnWidthInPx + (parseInt(window.getComputedStyle(newRegion, null).getPropertyValue("margin-right")));
        var numCols = Math.floor(availableBrowserWidth / availableRegionWidth);
        viewportIdx = 1;
        var colCount = 1;
        while (newRegion.msRegionOverflow === "overflow") {
            var newCol = getNewRegion(columnWidthInPx);
            if (viewportIdx > Math.floor(colCount / numCols)) {
                colContainer.appendChild(newCol);
            }
            else {
                viewportIdx++;
                colContainer = getNewViewportDiv();
                if (articleDirection === "rtl") {
                    colContainer.style.left = -1 * viewportIdx * document.documentElement.clientWidth + "px";
                }
                else {
                    colContainer.style.left = viewportIdx * document.documentElement.clientWidth + "px";
                }

                colContainer.appendChild(newCol);
                mainCont.appendChild(colContainer);
            }
            colCount++;
            newRegion = newCol;
        }

        while ((colCount % numCols) !== 0) {
            var newCol = getNewRegion(columnWidthInPx);
            colContainer.appendChild(newCol);
            colCount++;
        }

        return viewportIdx;
    }

    
    module.__proto__.paginate = function () {
        clearRegions();
        var viewportIndex = generateNewRegions();
        generateCarouselDots(viewportIndex);
        numViewport = viewportIndex + 1; 
    }

    
    module.__proto__.scrollRight = function () {
        var scrollFraction = 20;
        var timeInterval = 15;  
        var initScroll = document.getElementById("MainContainer").scrollLeft;
        if (scrollAllowed) {
            scrollAllowed = false;
            var intId = setInterval(function () {
                document.getElementById("MainContainer").scrollLeft += document.documentElement.clientWidth / scrollFraction;
                if (document.getElementById("MainContainer").scrollLeft >= (initScroll + document.documentElement.clientWidth)) {
                    document.getElementById("MainContainer").scrollLeft = (initScroll + document.documentElement.clientWidth);
                    clearInterval(intId);
                    scrollAllowed = true;
                }
            }, timeInterval);
        }
    }

    
    module.__proto__.scrollLeft = function () {
        var scrollFraction = 20;
        var timeInterval = 15;  
        var initScroll = document.getElementById("MainContainer").scrollLeft;
        if (scrollAllowed) {
            scrollAllowed = false;
            var intId = setInterval(function () {
                document.getElementById("MainContainer").scrollLeft -= document.documentElement.clientWidth / scrollFraction;
                if (document.getElementById("MainContainer").scrollLeft <= (initScroll - document.documentElement.clientWidth)) {
                    document.getElementById("MainContainer").scrollLeft = (initScroll - document.documentElement.clientWidth);
                    clearInterval(intId);
                    scrollAllowed = true;
                }
            }, timeInterval);
        }
    }

    
    module.__proto__.scrollToNextPage = function () {
        if (articleDirection === "rtl") {
            scrollLeft();
        }
        else {
            scrollRight();
        }

        highlightCarouselDot();
    }

    
    module.__proto__.scrollToPrevPage = function () {
        if (articleDirection === "rtl") {
            scrollRight();
        }
        else {
            scrollLeft();
        }

        highlightCarouselDot();
    }

    
    module.__proto__.adjustImageDimensions = function () {
        var dominantImageSrc = document.getElementById("dominantimage").getAttribute("src");
        var imageElemList = document.getElementsByClassName("__reading__mode__extracted__imagecontainer");

        
        var ifrmElem = document.getElementById("__rv_dataSource");
        var ifrm = (ifrmElem.contentWindow || ifrmElem.contentDocument);
        if (ifrm.document) {
            ifrm = ifrm.document;
        }

        var imageElemListIframe = ifrm.getElementsByTagName("img");

        for (var idx = 0; idx < imageElemListIframe.length; idx++) {
            if (imageElemListIframe[idx].getAttribute("src") == dominantImageSrc) {
                imageElemListIframe[idx].style.display = "none";
            }

            var maxColumnWidth = columnWidth * remBaseValue;
            var maxHeightFactor = 3 / 8;
            imageElemListIframe[idx].classList.remove("ienolimitimage");
            if ((imageElemListIframe[idx].naturalHeight) > (document.documentElement.clientHeight * maxHeightFactor)) {
                imageElemListIframe[idx].classList.add("prvheightlimitimage");
                if ((imageElemListIframe[idx].width >= maxColumnWidth)) {
                    imageElemListIframe[idx].classList.add("iewidthlimitimage");
                }
            }
            else if ((imageElemListIframe[idx].naturalWidth >= maxColumnWidth)) {
                imageElemListIframe[idx].classList.add("iewidthlimitimage");
            }
            else {
                imageElemListIframe[idx].classList.add("ienolimitimage");
            }
        }
    }

    
    module.__proto__.highlightCarouselDot = function () {
        var carouselPanel = document.getElementById("page_carousel");
        updateNavigationButtonsVisibility();
        if (carouselPanel.style.display != "none") {
            var currentPosition = Math.floor((document.getElementById("MainContainer").scrollLeft) / document.documentElement.clientWidth);
            var highlightedDot = carouselPanel.querySelector(".carousel_dots_checked");
            if (highlightedDot) {
                highlightedDot.classList.remove("carousel_dots_checked");
            }

            var allCarousels = carouselPanel.getElementsByClassName("carousel_dots");
            if ((allCarousels.length > 0) && (currentPosition < allCarousels.length)) {
                allCarousels[currentPosition].classList.add("carousel_dots_checked");
            }
        }
    }

    
    module.__proto__.generateCarouselDots = function (viewportCount) {
        var minViewportCount = 1;
        var maxViewportCount = 7;
        var carouselPanel = document.getElementById("page_carousel");
        if ((viewportCount >= minViewportCount) && (viewportCount < maxViewportCount)) {
            carouselPanel.style.display = "block";
            for (var i = 0; i <= viewportCount; i++) {
                var carouselDot = document.createElement("span");
                carouselDot.className = "carousel_dots";
                carouselPanel.appendChild(carouselDot);
            }

            highlightCarouselDot();
        }
        else {
            carouselPanel.style.display = "none";
        }
    }

    
    module.__proto__.addDropCaps = function () {
        var allParagraph = document.getElementsByTagName("p");
        for (var i = 0; i < allParagraph.length; i++) {
            if (allParagraph[i].innerText.length > 0) {
                if (articleDirection == "rtl") {
                    allParagraph[i].classList.remove("dropCaps");
                    allParagraph[i].classList.add("rtlDropCaps");
                }
                else {
                    allParagraph[i].classList.remove("rtlDropCaps");
                    allParagraph[i].classList.add("dropCaps");
                }
                break;
            }
        }
    }
    
    module.__proto__.prepareContentIframe = function () {
        var ifrmElem = document.getElementById("__rv_dataSource");
        var ifrm = (ifrmElem.contentWindow || ifrmElem.contentDocument);
        if (ifrm.document) ifrm = ifrm.document;
        ifrm.body.innerHTML = document.getElementById("htmlNonSecureZone").innerHTML;
        ifrmElem.style.msFlowInto = "main";
        ifrmElem.setAttribute("msallowfullscreen", "msallowfullscreen");
        ifrmElem.setAttribute("allowfullscreen", "allowfullscreen");
        document.getElementById("__reading__mode__mainbody__id").style.display = "none";
        document.getElementById("regionContainer").style.display = "block";
        ifrm.querySelector("html").setAttribute("dir", articleDirection);
        ifrm.body.classList.add("contentBody");
        ifrm.body.style.paddingTop = "0px";
        ifrmElem.style.display = "none";
    }

    
    module.__proto__.prepareAndPaginate = function () {
        calcuteTimeToRead();
        addDropCaps();
        
        if (document.documentElement.clientWidth >= minWidthForMultiColumn) {
            if (document.getElementById("title").innerHTML.length == 0) {
                document.getElementById("CoverPageProgress").style.display = "block";
            }
            else {
                document.getElementById("CoverPageProgress").style.display = "none";
                document.getElementById("navButtonContainer").style.display = "block";
                document.getElementById("end_mark_id").style.display = "block";
                getHostName();
            }

            adjustTablesLayout();
            styleCoverPage();
            prepareContentIframe();
            adjustImageDimensions();
            calcuteTimeToRead();
            updateNavigationButtonsVisibility();
            paginate();
        }
        else {
            getHostName();
            document.getElementById("CoverPageProgress").style.display = "none";
        }
    }

    
    module.__proto__.clearRegions = function () {
        var container = document.getElementById("regionContainer");
        while (container.firstChild) {
            container.removeChild(container.firstChild);
        }

        var carouselPanel = document.getElementById("page_carousel");
        while (carouselPanel.firstChild) {
            carouselPanel.removeChild(carouselPanel.firstChild);
        }
    }

    
    module.__proto__.rePaginate = function () {
        if (isPaginatedReadingViewInitialized == true) {
            var currentLeft = document.getElementById("MainContainer").scrollLeft;
            clearRegions();
            if (document.documentElement.clientWidth >= minWidthForMultiColumn) {
                prepareAndPaginate();
                document.getElementById("MainContainer").scrollLeft = currentLeft;
            }
            else {
                document.getElementById("__reading__mode__mainbody__id").style.display = "block";
                document.getElementById("__reading__mode__mainbody__id").style.padding = "0px";
                document.getElementById("navButtonContainer").style.display = "none";
                var header = document.getElementsByClassName("header_content");
                header[0].classList.remove("header_content_background");
            }
            return SCRIPT_PASSED;
        }
        return SCRIPT_FAILED;
    }

    
    module.__proto__.afterResize = function () {
        rePaginate();
    }

    
    module.__proto__.updateDirection = function () {
        articleDirection = document.querySelector("html").dir;
    }

    
    module.__proto__.styleCoverPage = function () {
        
        var dominantImage = document.getElementById("dominantimage");
        if ((dominantImage.getAttribute("src").length == 0) && (document.getElementById("title").innerHTML.length != 0)) {
            dominantImage.style.display = "none";
            document.getElementById("header_skew_section_id").className = "header_skew_section";
            document.getElementById("header_content_id").classList.remove("header_content");
            if (articleDirection === "rtl") {
                document.getElementById("header_section_id").className = "header_section_no_image_rtl";
                document.getElementById("header_content_id").className = "header_content_no_image_rtl";
                document.getElementById("header_pane_id").className = "header_pane_no_image_rtl";
            }
            else {
                document.getElementById("header_section_id").className = "header_section_no_image";
                document.getElementById("header_content_id").className = "header_content_no_image";
                document.getElementById("header_pane_id").className = "header_pane_no_image";
            }
        }
        else if ((dominantImage.naturalWidth >= (document.documentElement.clientWidth) * 0.9) && (dominantImage.naturalHeight >= (document.documentElement.clientHeight) * 0.9)) {
            
            dominantImage.style.display = "block";
            document.getElementById("dominantimage_container").classList.add("__reading__mode__extracted__dominant__image_full");
            dominantImage.style.height = "100%";
            dominantImage.style.width = "100%";
            dominantImage.style.maxWidth = "100%";

            document.getElementById("header_section_id").className = "header_section_full_page_image";
            document.getElementById("header_skew_section_id").className = "header_skew_section";
            document.getElementById("header_content_id").classList.remove("header_content");
            document.getElementById("header_content_id").className = "header_content_full_page_image";
            document.getElementById("header_pane_id").className = "header_pane_full_page_image";
        }
        else {
            dominantImage.style.display = "block";
            dominantImage.style.maxWidth = "100%";
            var header = document.getElementsByClassName("header_content");
            header[0].classList.remove("header_content_background");
            header[0].classList.remove("header_content_no_image");
            document.getElementsByClassName("__reading__mode__extracted__title")[0].classList.remove("__reading__mode__extracted__title_negative_skew");
            document.getElementsByClassName("__reading__mode__bylineentries")[0].classList.remove("__reading__mode__bylineentries_negative_skew");
            dominantImage.style.marginLeft = "0px";
            if (articleDirection == "rtl") {
                document.getElementById("dominantimage_container").style.left = "50%";
                header[0].style.left = "0";
                header[0].style.width = "50%";
                dominantImage.style.marginRight = "0px";
            }
        }
    }

    
    module.__proto__.calcuteTimeToRead = function () {
        var currentLanguage = document.querySelector("html").lang;
        if (currentLanguage && currentLanguage.indexOf("en") === 0) {
            var avgReadingSpeed = 250; 
            var avgSpeedForImage = 4; 
            var contentElement = document.getElementById("content");
            var timeToReadSec = Math.floor((60 * contentElement.innerText.split(/\s+/).length) / avgReadingSpeed);
            var numImages = document.getElementsByTagName("img").length;
            timeToReadSec += (numImages * avgSpeedForImage);
            if (timeToReadSec >= 60) {
                var timeToReadMin = Math.floor(timeToReadSec / 60);
                if ((typeof timeToReadText.LocalizedMin != 'undefined') && (typeof timeToReadText.LocalizedMins != 'undefined') && (typeof timeToReadText.LocalizedRead != 'undefined')) {
                    if (timeToReadMin == 1) {
                        document.getElementById("timeToRead").innerHTML = "<span dir=\"ltr\" title=\"Estimated average time to read\">" + timeToReadMin + " " + timeToReadText.LocalizedMin + " " + timeToReadText.LocalizedRead + "</span>";
                    }
                    else {
                        document.getElementById("timeToRead").innerHTML = "<span dir=\"ltr\" title=\"Estimated average time to read\">" + timeToReadMin + " " + timeToReadText.LocalizedMins + " " + timeToReadText.LocalizedRead + "</span>";
                    }
                }
            }
        }
    }

    
    module.__proto__.getHostName = function () {
        var currentUrl = window.location.pathname; 
        var node = document.createElement("a");
        node.href = currentUrl;
        document.getElementById("source").innerText = node.hostname;
    }

    
    module.__proto__.initializeIframe = function () {
        var ifrmElem = document.getElementById("__rv_dataSource");
        var htmlsrc = '<!DOCTYPE html><html><head><meta http-equiv="Content-Security-Policy" content="script-src \'self\'"><link href="ms-appx-web:///assets/ReadingView/prv/css/StyleNormalPRV.css" rel="stylesheet"><link href="ms-appx-web:///assets/ReadingView/prv/css/BaseStylePRV.css" rel="stylesheet"><link href="ms-appx-web:///assets/ReadingView/prv/css/BaseFontSizePRV.css" rel="stylesheet"></head><body></body></html>';
        ifrmElem.contentWindow.document.open('text/html', 'replace');
        ifrmElem.contentWindow.document.write(htmlsrc);
        ifrmElem.contentWindow.document.close();
    }

    
    module.__proto__.addBaseTagToIframe = function () {
        var ifrmElem = document.getElementById("__rv_dataSource");
        var ifrm = (ifrmElem.contentWindow || ifrmElem.contentDocument);
        if (ifrm.document) ifrm = ifrm.document;
        var baseTag = ifrm.getElementsByTagName("base");
        if (baseTag.length == 0) {
            var bt = ifrm.createElement("base");
            bt.setAttribute("href", document.getElementById("content_base").getAttribute("href"));
            bt.setAttribute("target", "_top");
            ifrm.getElementsByTagName("head")[0].appendChild(bt);
        }
    }

    
    module.__proto__.setLocalizedStrings = function (inputJson) {
        
        timeToReadText = JSON.parse(inputJson);
    }

    
    module.__proto__.getNumPages = function () {
        return (numViewport > 0) ? numViewport : 1;
    }

    
    module.__proto__.getBrowserWidth = function () {
        return document.documentElement.clientWidth;
    }

    
    module.__proto__.getBrowserHeight = function () {
        return document.documentElement.clientHeight;
    }

    
    module.__proto__.updateNavigationButtonsVisibility = function () {
        var currPageindex = Math.floor((document.getElementById("MainContainer").scrollLeft) / document.documentElement.clientWidth);
        document.getElementById("leftButton").style.display = "block";
        document.getElementById("rightButton").style.display = "block"
        if (articleDirection === "ltr") {
            if (currPageindex == 0) {
                document.getElementById("leftButton").style.display = "none";
            }
            else if (currPageindex == (numViewport - 1)) {
                document.getElementById("rightButton").style.display = "none"
            }
        }
        else {
            if (currPageindex == 0) {
                document.getElementById("rightButton").style.display = "none";
            }
            else if (currPageindex == (numViewport - 1)) {
                document.getElementById("leftButton").style.display = "none"
            }
        }
    }

    
    module.__proto__.gotoHome = function () {
        document.getElementById("MainContainer").scrollLeft = 0;
        highlightCarouselDot();
    }

    
    module.__proto__.gotoEnd = function () {
        var numOfPages = getNumPages();
        document.getElementById("MainContainer").scrollLeft = (numOfPages - 1) * document.documentElement.clientWidth;
        highlightCarouselDot();
    }

    
    module.__proto__.initiatePaginatedReadingView = function (inputJson) {
        
        document.getElementById("MainContainer").addEventListener("scroll", highlightCarouselDot);
        var nextPage = document.getElementById("rightButton");
        nextPage.addEventListener("click", scrollToNextPage);

        var prevPage = document.getElementById("leftButton");
        prevPage.addEventListener("click", scrollToPrevPage);
        window.addEventListener("resize", afterResize);

        window.addEventListener("keydown", function (event) {
            if (event.keyCode === 36) {  
                event.preventDefault();
                gotoHome();
            }
            else if (event.keyCode === 35) {  
                event.preventDefault();
                gotoEnd();
            }
        });
        
        var coverProgress = document.getElementById("CoverPageProgress");

        var coverProgressObserver = new MutationObserver(function (mutations) {
            mutations.forEach(function (mutation) {
                if (document.getElementById("title").innerHTML.length >= 0) {
                    coverProgress.style.display = "none";
                    document.getElementById("navButtonContainer").style.display = "block";
                    getHostName();
                }
            });
        });

        var config = { attributes: true, childList: true, characterData: true };

        coverProgressObserver.observe(document.getElementById("title"), config);
        try {
            setLocalizedStrings(inputJson);
            updateDirection();
            addBaseTagToIframe();
            prepareAndPaginate();
            isPaginatedReadingViewInitialized = true;
        }
        catch (err) {
            return SCRIPT_EXCEPTION_OCCURED;
        }
        return SCRIPT_PASSED;
    }

    return module;
})(ReadingModeViewer || {});

window.initializeIframe = ReadingModeViewer.initializeIframe;
window.initiatePaginatedReadingView = ReadingModeViewer.initiatePaginatedReadingView;
window.rePaginate = ReadingModeViewer.rePaginate;
window.getNumPages = ReadingModeViewer.getNumPages;
window.getBrowserWidth = ReadingModeViewer.getBrowserWidth;
window.getBrowserHeight = ReadingModeViewer.getBrowserHeight;
